// environment.h
//
// Created 1998 by Jouni K Seppnen
//
// $Id: environment.h,v 1.6 1999/08/09 12:29:45 jks Exp $
//

#ifndef ENVIRONMENT_H
#define ENVIRONMENT_H

#include <string>
using namespace std;

/** This class provides information from the environment.
    An object of the class gives access to environment variables,
    falling back to compile-time defaults if necessary. */
class environment {
    /** An enumeration of known environment variables. 
        N_ENV_VAR is a sentinel value. */
    enum env_var { ENV_LIBRARY, ENV_INCLUDE, ENV_TEMPLATE, ENV_N_VAR };
    /// The names of known environment variables.
    static const char * const names[ENV_N_VAR];
    /// The default values of known environment variables.
    static const char * const defaults[ENV_N_VAR];
    /// The values of known environment variables.
    string *values[ENV_N_VAR];
public:
    /// Constructor
    environment();
    /// Destructor
    ~environment();
    /** Gives the value of BASSIST_LIBRARY, which is the full path
        name to the Bassist library. */
    const string &bassist_library() const { return *values[ENV_LIBRARY]; };
    /** Gives the value of BASSIST_INCLUDE_DIR, which is where the
        installed include files of the Bassist library are located. */
    const string &bassist_include_dir() const { return *values[ENV_INCLUDE]; };
    /** Gives the value of BASSIST_TEMPLATE_DIR, which is where the
        installed code template files are located. */
    const string &bassist_template_dir() const { return *values[ENV_TEMPLATE]; };
};

#endif // ENVIRONMENT_H

// Local Variables:
// mode: c++
// make-backup-files: t
// c-basic-offset: 4
// End:
