// function.h
//
// Created 1998 by Karri-Pekka Laakso
//
// $Id: function.h,v 1.3 1999/08/04 12:05:04 jks Exp $
//

#ifndef FUNCTION_H
#define FUNCTION_H

#include <string>
using namespace std;

#include "node.h"

/** The function prototype class. 
   
   The prototypes of the user-defined functions are represented as
   members of this class. During the code generation, the prototype
   definitions are written to a header file, which is included in all
   of the generated code files. 

   The name of the function and the number of its arguments define
   completely the function, because all the user defined functions
   take double type arguments and return a double value.
  */
class function : public parseTreeNode {
    /// The name of the function.
    string name;
    /// The number of the arguments.
    int args;
    
public:
    /** The constructor.
       @param n The name of the function.
       @param a The number of the arguments. */
    function(const string &n, int a) : name(n), args(a) {}
    
    /// Return the function name.
    inline const string &giveName() { return name; }
    /// Return the number of arguments.
    inline int giveArgs() { return args; }

    /** Model syntax reprinting function, prints to stdout.
        
       @see node::printCode
       @see compiler::printTree */
    void printCode();
};

#endif // FUNCTION_H

//-----------------------------//
// Local Variables:            //
// mode: c++                   //
// make-backup-files: t        //
// End:                        //
//-----------------------------//
