%{
// lex-codegen.l
//
// Created 1998 by Karri-Pekka Laakso
//
// $Modified: Fri Aug 21 15:39:35 1998 kpalaaks$
// $Id: lex-codegen.l,v 1.11 1999/10/26 10:19:41 jks Exp $
//

#include <string>
using namespace std;

#include <stdio.h>
#include "parse-codegen.h"
#include "scope.h"
#include "main.h"

// Input stack for multiple input buffers    
YY_BUFFER_STATE input_stack[CG_MAX_SCOPES];

#define SAVECODE s->saveLoopCode(yytext, yyleng)

// ECHO saves text to loop buffers and puts it to output, if wanted
#define ECHO {\
    s->saveLoopCode(yytext, yyleng);\
    if (s->echoOn())\
	fwrite(yytext, yyleng, 1, yyout);\
}

%}


%option 8bit
%option yylineno
%option noyywrap
%x lstart inv ind 

ID	[A-Z][A-Z0-9_]*	
RULE    NAME|VNAME|TYPE|CREATE|POINTER|DEFINITION|ADD_TO_LIST|DEPENDENCIES|INIT|REC_DEFINITION|REC_PARAMETER|REC_INIT|DIST|DISTPVAL|VALFUNCTION|DEPFUNCTION|IMPLICIT|RECNAME|DEPENDENCYHACK_H|DEPENDENCYHACK_C|INITIALIZER|PROPOSER|PROPOSALDENSITY|BASE|AG_IMPL_PROTO|AG_IMPL_CODE
%%

^\/\/" "FOREACH		{ ECHO; BEGIN(lstart); return FOREACH; }

<lstart>[ \t]*		{ ECHO; } 
<lstart>{ID}		{ ECHO; cglval.s = new string(yytext); return ID; }
<lstart>\)[ \t]*\n 	{ ECHO; BEGIN(INITIAL); return *yytext; }
<lstart>.		{ ECHO; return *yytext; }

^\/\/" "?ENDFOR		{
			    bool scopeAtEnd = s->loopEnd(yytext);
			    
			    if (!scopeAtEnd) {
				input_stack[s->giveId()] = YY_CURRENT_BUFFER;
				yy_switch_to_buffer(yy_scan_string(s->giveBuffer().c_str()));
			    } else {
				if (s->previous()->echoOn())
				    fprintf(yyout, "// ENDFOR");
				return ENDFOR;
			    }
			}


<<EOF>> 		{
			    if (s->giveId() >= 1) {
				yy_delete_buffer(YY_CURRENT_BUFFER);
				bool scopeAtEnd = s->loopEnd(yytext);
				
				if (!scopeAtEnd)
				    yy_switch_to_buffer(yy_scan_string(s->giveBuffer().c_str()));
				else {
				    yy_switch_to_buffer(input_stack[s->giveId()]);
				    if (s->previous()->echoOn())
					fprintf(yyout, "// ENDFOR");
				    return ENDFOR;
				}
			    } else {
				cglineno = 0;
				yyterminate();
			    }
			}


\$/{ID}\.{RULE}		SAVECODE; BEGIN(inv);
<inv>{ID}/\.		{ SAVECODE; cglval.s = new string(yytext); return VAR; }
<inv>\.			{ SAVECODE; BEGIN(ind); return *yytext; } 
<ind>{RULE}		{ SAVECODE; BEGIN(INITIAL); cglval.s = new string(yytext); return FUNC; }

"@USERFUNCDEFS"		{ return USERFUNCDEFS; }
"@VERSION" 		{ return VERSION; }
"@MODELDEF"	 	{ return MODELDEF; }
"@MODEL"	 	{ return MODEL; }
"@DATE"	 		{ return DATE; }
"@BASSIST_INCLUDE_DIR"  { return INCLUDE_DIR; }
"@BASSIST_LIBRARY"      { return LIBRARY; }
"@PROGRAM"              { return PROGRAM; }
"@USERCFLAGS"           { return USERCFLAGS; }
"@OWNDISTRIBUTIONS_H"   { return OWNDISTS_H; }
"@OWNDISTRIBUTIONS_CC"  { return OWNDISTS_CC; }

.			ECHO;


%%

//-----------------------------//
// Local Variables:            //
// mode: c++                   //
// make-backup-files: t        //
// End:                        //
//-----------------------------//
