#ifndef DEBUG_H
#define DEBUG_H

#include "ctype.h"
#include <string>
#include <cstring>
using namespace std;

#include "compiler-extensions.h"

/** @name The debugging routines */
//@{

/** Sets the debug flags according to the command line.
    Only does anything if BASSIST_DEBUG is defined.
    @param flags The debug flags to be enabled. */
inline void debugInit(string);
/** Tests whether the given debug flag is enabled.
    Only does anything if BASSIST_DEBUG is defined, otherwise returns false.
    @param flag The debug flag.
    @return true if the flag is enabled */
inline bool debugIsEnabled(char);
/** Prints a debug message if the given debug flag is enabled.
    Only does anything if BASSIST_DEBUG is defined.
    @param flag The debug flag.
    @param format The debug message format like in printf().
    @param ... The variable list of arguments for the print format. */
inline void DEBUG(char, char *, ...) GCC_FORMAT(printf,2,3);

#ifdef BASSIST_DEBUG
inline void 
debugInit(string flags)
{
    extern char *enabledDebugFlags;

    if (enabledDebugFlags)
	delete[] enabledDebugFlags;
    enabledDebugFlags = new char[flags.length() + 1];
    strcpy(enabledDebugFlags, flags.c_str());

    // For case-insensitivity, lowercase the string
    for (char *p = enabledDebugFlags; *p != 0; p++)
        *p = tolower(*p);
}
#else
inline void
debugInit(string)
{
}
#endif

#ifdef BASSIST_DEBUG
inline bool 
debugIsEnabled(char flag)
{
    extern char *enabledDebugFlags;
    return enabledDebugFlags
	&& (strchr(enabledDebugFlags, tolower(flag)) 
	    || strchr(enabledDebugFlags, '+'));
}
#else
inline bool
debugIsEnabled(char)
{
    return false;
}
#endif

#ifdef BASSIST_DEBUG
inline void 
DEBUG(char flag, char *format, ...)
{
    if (debugIsEnabled(flag)) {
	va_list ap;
	printf("# ");
	va_start(ap, format);
	vfprintf(stdout, format, ap);
	va_end(ap);
	fflush(stdout);
    }  
}
#else
inline void
DEBUG(char, char *, ...)
{
}
#endif

//@}


#endif // DEBUG_H

// Local Variables:
// mode: c++
// c-basic-offset: 4
// case-replace: nil
// make-backup-files: t
// End:
