// common.h
//
// Created 1997 by Hannu Toivonen
//
// $Id: common.h,v 1.11 1999/10/06 07:39:35 jks Exp $
//

#ifndef COMMON_H
#define COMMON_H

#include <cstdlib>
#include <cstdio>
#include <cstdarg>
//#include <cstring>
//#include <string>
using namespace std;

#include "mt-rng.h"
#include "compiler-extensions.h"

#include "bdebug.h"

typedef double logprob;
typedef double realval;
typedef double intval;

extern int iteration;

const double BASS_INFINITY = 1e10;

inline void
ASSERT_I(bool x, char *file, int line)
{
  if (!x) { 
    printf("\nInternal error: file '%s', line %d\n", file, line); 
    exit(1); 
  } 
}
#define ASSERT(x) ASSERT_I((x), __FILE__, __LINE__)

//----------------------------------------
// Mathematics
//----------------------------------------

// /// Returns a random number in [a, y].
// inline double randH(double a, double y) {
//     ASSERT(y-a != 0);
//     return (a + (y-a) * ((1.0*rand())/RAND_MAX));
// }

extern class MT_RandomNumberGenerator rng;
inline double randH(double a, double b) {
    ASSERT(b-a != 0);
    return rng.random(a,b);
}

inline double max(double x, double y) { return x<y ? y : x; }
inline double min(double x, double y) { return x>y ? y : x; }

// class hashStringTable
//  A hash implementation for void pointers.
//
// 

/**
	Class hashStringTable. This class provides traditional
   	interface to hash items except that size of the table
   	is constant. Items are hashed according
   	to their names. If collision occurs, the item is hashed
   	to the next available place.

	Simulation has two instances of hashStringTable
	to store parameters and constants. In addition
	every record in the model has a hashStringTable.
*/

class hashStringTable {
protected:
    /// Maximum size of hash string table.
    static const unsigned int HASH_CONST = 100003;

    /** 
    	Names of the hashed entities.
    */
    char *array[HASH_CONST];
    
    /** 
	Value id[x] corresponds sequence number of the item
	that was hashed to the index x.
    */
    int id[HASH_CONST];
   
    /** 
    	Pointers to the hashed items.
    */ 
    void *content[HASH_CONST];

    /**
    	 Counter that is increased every time, when 
    	 new item is added to this hashStringTable.
    */
    int count;
    
    /**
   	Hash key for an entity i is generated according to 
   	its name as follows
   	\TEX{$key_{i} = (\prod_{i=1}^{length(name)} name[i])$} mod HASH-CONST.     	
	@return Hash key for a given name.
    */
    unsigned int hash(char *string);

    /**
    	Index according to which next item will be 
    	searched.
    */
    unsigned int nextItem;
    
public:
    /**
       Constructor.
      */
    hashStringTable() : count(0), nextItem(0) {};

    /**
       @param *string Parameter that will be hashed to this hashStringTable
       @param *
       @return
      */
      
    int add(char *string, void *pointer);

    /**
    	Returns pointer to the item, whose name is given parameter.
    	@param *string Name of the item
    	@return Pointer to the item.
    */
    void *get(char *string);

    /**
    	Returns pointer to first item in this hashStringTable.
    */
    void *first();

    /**
    	Returns pointer to the next item in this hashStringTable 
    	according to index nextItem.
    */
    void *next();
};



#endif // COMMON_H

// Local Variables:
// mode: c++
// c-basic-offset: 4
// case-replace: nil
// make-backup-files: t
// End:
