// constant.h
//
// Created 1997 by Karri-Pekka Laakso
//
// @(#)$Id: constant.h,v 1.5 1999/08/10 08:29:25 jks Exp $
//

#ifndef CONSTANT_H
#define CONSTANT_H

#include "quantity.h"
#include "scanner.h"

// Forward declaration
class parameter;

/**
	Constants are non-stochastic quantities that have a fixed
	value. It is obvious, that there is no need to know the dependents of 
	an instance of constant, because its value is fixed in the constructor.
	Therefore the implementation of the addDependent method is empty.
	The implementation of the class constant is simple.
	Value of constant is achievable via method value().

	@author Karri-Pekka Laakso

*/
class constant : public quantity {
protected:
    /// Current value of this constant
    double currentValue;

    /** Indicates if the value of this constant has been defined
	either in the model or in the data. */
    bool valueSet;
    
public:
    /** The constructor for the constants whose values are to be
	defined in the data. 
	@param name of this constant
      */
    constant(const string &name) :
	quantity(name), currentValue(0), valueSet(false) { };

    /** The constructor for the constants whose values are defined in
	the model. 
	@param name of this constant
	@param value of this constant
      */
    constant(const string &name, double value) :
	quantity(name), currentValue(value), valueSet(true) { };

    /** Empty implementation. There is no need to know the dependents,
	because the constants are not simulated. */
    void addDependent(parameter *) { };
    
    /**
       Returns the value of this constant
       @return current value of this constant
    */
    inline double value() const { return currentValue; }

    /** Tests, if the value of this constant has been defined either
	in the model or in the data. If not, this constant is
	undefined, and an error occurs. */
    void initialize();
    
    /**
	Assigns this constant a value, if the string "name = value"
	is found from the datafile. 
       	@param scanner Scanner
       	@param name Name of the constant
       	@param init 
       	@return True if the sequence was found, false otherwise.
      */
    virtual bool askValue(scanner &sc, const char *n = 0, bool init = false);
};


#endif // CONSTANT_H

//------------------------------------------------
// Local Variables:				//
// mode: c++					//
// make-backup-files: t				//
// End:						//
//------------------------------------------------

