%{
// parse-codegen.y
//
// Created 1998 by Karri-Pekka Laakso
//
// $Modified: Mon Aug  3 15:13:02 1998 kpalaaks$
// $Id: parse-codegen.y,v 1.6 1999/10/26 10:19:41 jks Exp $
//

#include <cstdio>
#include <ctime>
#include <cstring>
#include <string>
using namespace std;

#include "main.h"
#include "scope.h"
#include "distribution.h"
    
int yyerror(char *msg)
{
  extern int cglineno;
  extern char *cgtext;

  fprintf(stderr, "%s:%d: %s -- token %s\n", c.infilename.c_str(), cglineno, msg, cgtext);
  return 0;
}


int yylex(void);

string escapeAndQuote(string in);

%}

%start lines

%union {
  string *s;
}

%token <s> ID VAR FUNC 
%token FOREACH ENDFOR USERFUNCDEFS VERSION MODELDEF MODEL DATE INCLUDE_DIR LIBRARY PROGRAM 
%token USERCFLAGS OWNDISTS_H OWNDISTS_CC

%%

lines:	lines foreach
	| lines USERFUNCDEFS 	{
			            vector<function*>::iterator
					f = c.functions->begin(),
					end = c.functions->end();
				    for (; f != end; f++) {
					(*f)->printCode();
					fprintf(c.file, ";\n");
				    }
                                }
	| lines VERSION		{ fprintf(c.file, "%s", c.version().c_str()); }
	| lines MODELDEF	{ fprintf(c.file, "%s", 
					  escapeAndQuote(c.modelDefinition()).c_str()); }
	| lines MODEL		{ fprintf(c.file, "%s", c.modelName().c_str()); }
	| lines DATE		{ time_t t = time(0); char tm[81];
				  strncpy(tm, asctime(localtime(&t)), 80);
				  tm[strlen(tm)-1] = '\0';
				  fprintf(c.file, "%s", tm); }
        | lines INCLUDE_DIR     { fprintf(c.file, "%s", env.bassist_include_dir().c_str()); }
	| lines LIBRARY         { fprintf(c.file, "%s", env.bassist_library().c_str()); }
	| lines PROGRAM         { fprintf(c.file, "%s", c.prog.c_str()); }
	| lines USERCFLAGS      { fprintf(c.file, "%s", c.usercflags.c_str()); }
        | lines OWNDISTS_H      { distributionDefinition::printHeaderFile(); }
	| lines OWNDISTS_CC     { distributionDefinition::printCodeFile(); }
	| /* empty */
	;

foreach: FOREACH ID '(' ID ')'	{ s = new scope(*$2, *$4); delete $2; delete $4; }
	 inserts ENDFOR		{ scope *prev = s->previous(); delete s; s = prev; }
	;

inserts: inserts insert
	| /* empty */		
	;

insert: VAR '.' FUNC		{ s->insertText(*$1, *$3); delete $1; delete $3; }
	| foreach
	;

%%

string 
escapeAndQuote(string in)
{
    string out = "\"";
    for (string::iterator p = in.begin(); p != in.end(); p++) {
	switch(*p) {
	case '\'': 
	case '\"': 
	case '\\':
	    out += "\\" + *p;
	    break;
	case '\n':
	    out += "\\n\"\n\"";
	    break;
	default:
	    out += *p;
	    break;
	}
    }

    out += "\"";
    return out;
}

// Local Variables:
// mode: c++
// c-basic-offset: 4
// case-replace: nil
// make-backup-files: t
// End:
